#include <DAQ_Configurator.h>
#include <tclap/CmdLine.h>

#include <TCP_Protocol_Configurator.h>
#include <UDP_Protocol_Configurator.h>
#include <Kafka_Provider_Configurator.h>
#include <UDP_Provider_Configurator.h>
#include <Redis_Provider_Configurator.h>
#include <File_Archiver_Configurator.h>
#include <Astri_Monitor_Configurator.h>
#include <Astri_Checker_Configurator.h>


#define PACKETS_PATH std::string(std::getenv("BIAS_PREFIX")).append("/config/packets/xml_packets/")

DAQConfigurator::DAQConfigurator(){
	setProtocol("udp");
	setPacket("astri_tm");
	setArchiverType("file");
	setProviderType("kafka");
	setCheckerType("astri");
	setMonitor("astri");
}

DAQConfigurator::DAQConfigurator(int argc, char ** argv){
	try {
		TCLAP::CmdLine cmd("Usage Description WIP", ' ', "1");
		cmd.ignoreUnmatched(true);
		TCLAP::ValueArg<std::string> protocol("","protocol","communication protocol",false,"udp","[tcp|udp|.. |supported protocols]");
		cmd.add( protocol );
		TCLAP::ValueArg<std::string> packet("","packet","the file name of the packet structure",false,"astri_tm","[astri_tm|..]");
		cmd.add( packet );
		TCLAP::ValueArg<std::string> archiver("","archiver","archiver destination",false,"file","[file|kafka]");
		cmd.add( archiver );
		TCLAP::ValueArg<std::string> provider("","provider","provider destination",false,"file","[file|kafka]");
		cmd.add( provider );
		TCLAP::ValueArg<std::string> checker("","checker","the checker class",false,"astri","[astri|..]");
		cmd.add( checker );
		TCLAP::ValueArg<std::string> monitor("","monitor","the packet monitor",false,"astri","[astri|..]");
		cmd.add( monitor );
		cmd.parse( argc, argv );

		setProtocol(protocol.getValue());
		setPacket(packet.getValue());
		setArchiverType(archiver.getValue());
		setProviderType(provider.getValue());
		setCheckerType(checker.getValue());
		setMonitor(monitor.getValue());

	}catch (TCLAP::ArgException &e){  // catch any exceptions
		std::cerr << "error: " << e.error() << " for arg " << e.argId() << std::endl;
		exit(EXIT_FAILURE);
	}
}

BaseArchiver * DAQConfigurator::createArchiver(int argc, char ** argv){
	if(getArchiverType().compare("file") == 0){
		FileArchiverConfigurator conf(argc,argv);
		return conf.createArchiver();
	}
	throw std::invalid_argument("Invalid archiver");
}

BaseProvider * DAQConfigurator::createProvider(int argc, char ** argv){
	if(getProviderType().compare("kafka") == 0){
		KafkaProviderConfigurator conf(argc,argv);
		return conf.createProvider();
	}
	if(getProviderType().compare("udp") == 0){
		UDPProviderConfigurator conf(argc,argv);
		return conf.createProvider();
	}
	if(getProviderType().compare("redis") == 0){
		RedisProviderConfigurator conf(argc,argv);
		return conf.createProvider();
	}
	throw std::invalid_argument("Invalid provider");
}


BasePacket * DAQConfigurator::createPacket(int argc, char ** argv){
	std::string packet_path(PACKETS_PATH);
	packet_path.append(getPacket().append(".xml"));
	return new AstriPacket(packet_path);
}

BaseMonitor * DAQConfigurator::createMonitor(int argc, char ** argv){
	if(getMonitor().compare("astri") == 0){
		AstriMonitorConfigurator conf(argc,argv);
		return conf.createMonitor();
	}
	throw std::invalid_argument("Invalid Monitor");
}

BaseChecker * DAQConfigurator::createChecker(int argc, char ** argv){
	if(getMonitor().compare("astri") == 0){
		AstriCheckerConfigurator conf(argc,argv);
		return conf.createChecker();
	}
	throw std::invalid_argument("Invalid Monitor");
}

BaseProtocol * DAQConfigurator::createProtocol(int argc, char ** argv){
	if(getProtocol().compare("tcp") == 0){
		TCPProtocolConfigurator conf(argc,argv);
		return conf.createProtocol();
	}
	if(getProtocol().compare("udp") == 0){
		UDPProtocolConfigurator conf(argc,argv);
		return conf.createProtocol();
	}
	throw std::invalid_argument("Invalid protocol");
}
